@@ -68,18 +68,18 @@ as.promise.mirai <- function(x) {
 
   if (is.null(promise)) {
 
-    promise <- promises::then(
-      promises::promise(
-        function(resolve, reject)
-          context <- set_promise_context(x, environment())
-      ),
-      onFulfilled = function(value)
-        if (is_error_value(value) && !is_mirai_interrupt(value))
-          stop(if (is_mirai_error(value)) value else nng_error(value)) else
-            value
-    )
-
-    if (!unresolved(x)) {
+    if (unresolved(x)) {
+      promise <- promises::then(
+        promises::promise(
+          function(resolve, reject)
+            context <- set_promise_context(x, environment())
+        ),
+        onFulfilled = function(value)
+          if (is_error_value(value) && !is_mirai_interrupt(value))
+            stop(if (is_mirai_error(value)) value else nng_error(value)) else
+              value
+      )
+    } else {
       value <- .subset2(x, "value")
       promise <- if (is_error_value(value) && !is_mirai_interrupt(value))
         promises::promise_reject(if (is_mirai_error(value)) value else nng_error(value)) else
